@@ -12,7 +12,7 @@ from typing import Any, List, Tuple, Union
 from .errors import ParseError
 
 _INT_RE = r"(0|-?[1-9]\d*)"  # A decimal int, don't allow "-0".
-PAGE_RANGE_RE = "^({int}|({int}?(:{int}?(:{int}?)?)))$".format(int=_INT_RE)
+PAGE_RANGE_RE = f"^({_INT_RE}|({_INT_RE}?(:{_INT_RE}?(:{_INT_RE}?)?)))$"
 # groups:         12     34     5 6     7 8
 
 
@@ -31,7 +31,6 @@ class PageRange:
       -  to_slice() gives the equivalent slice.
       -  str() and repr() allow printing.
       -  indices(n) is like slice.indices(n).
-
     """
 
     def __init__(self, arg: Union[slice, "PageRange", str]) -> None:
@@ -43,6 +42,7 @@ class PageRange:
             where the brackets indicate optional ints.
         Remember, page indices start with zero.
         Page range expression examples:
+
             :     all pages.                   -1    last page.
             22    just the 23rd page.          :-1   all but the last page.
             0:3   the first three pages.       -2    second-to-last page.
@@ -68,7 +68,7 @@ class PageRange:
         m = isinstance(arg, str) and re.match(PAGE_RANGE_RE, arg)
         if not m:
             raise ParseError(arg)
-        elif m.group(2):
+        if m.group(2):
             # Special case: just an int means a range of one page.
             start = int(m.group(2))
             stop = start + 1 if start != -1 else None
@@ -82,10 +82,11 @@ class PageRange:
         True if input is a valid initializer for a PageRange.
 
         Args:
-          input: A possible PageRange string or a PageRange object.
+            input: A possible PageRange string or a PageRange object.
 
         Returns:
-          True, if the ``input`` is a valid PageRange.
+            True, if the ``input`` is a valid PageRange.
+
         """
         return isinstance(input, (slice, PageRange)) or (
             isinstance(input, str) and bool(re.match(PAGE_RANGE_RE, input))
@@ -114,20 +115,21 @@ class PageRange:
 
     def indices(self, n: int) -> Tuple[int, int, int]:
         """
-        Assuming a sequence of length n, calculate the start and stop
-        indices, and the stride length of the PageRange.
+        Assuming a sequence of length n, calculate the start and stop indices,
+        and the stride length of the PageRange.
 
         See help(slice.indices).
 
         Args:
-          n:  the length of the list of pages to choose from.
+            n:  the length of the list of pages to choose from.
 
         Returns:
-          Arguments for range()
+            Arguments for range().
+
         """
         return self._slice.indices(n)
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         if not isinstance(other, PageRange):
             return False
         return self._slice == other._slice
@@ -160,30 +162,33 @@ def parse_filename_page_ranges(
     Given a list of filenames and page ranges, return a list of (filename, page_range) pairs.
 
     Args:
-      args: A list where the first element is a filename. The other elements are
-        filenames, page-range expressions, slice objects, or PageRange objects.
-        A filename not followed by a page range indicates all pages of the file.
+        args: A list where the first element is a filename. The other elements are
+            filenames, page-range expressions, slice objects, or PageRange objects.
+            A filename not followed by a page range indicates all pages of the file.
 
     Returns:
-      A list of (filename, page_range) pairs.
+        A list of (filename, page_range) pairs.
+
     """
     pairs: List[Tuple[str, PageRange]] = []
-    pdf_filename = None
+    pdf_filename: Union[str, None] = None
     did_page_range = False
-    for arg in args + [None]:
+    for arg in [*args, None]:
         if PageRange.valid(arg):
             if not pdf_filename:
                 raise ValueError(
                     "The first argument must be a filename, not a page range."
                 )
 
+            assert arg is not None
             pairs.append((pdf_filename, PageRange(arg)))
             did_page_range = True
         else:
-            # New filename or end of list--do all of the previous file?
+            # New filename or end of list - use the complete previous file?
             if pdf_filename and not did_page_range:
                 pairs.append((pdf_filename, PAGE_RANGE_ALL))
 
+            assert not isinstance(arg, PageRange), arg
             pdf_filename = arg
             did_page_range = False
     return pairs
