@@ -5,7 +5,8 @@ from copy import deepcopy
 from io import BytesIO
 from pathlib import Path
 from random import shuffle
-from typing import List, Tuple
+from typing import Any, List, Tuple
+from unittest import mock
 
 import pytest
 
@@ -39,8 +40,7 @@ def get_all_sample_files():
         return {"data": []}
     with open(meta_file) as fp:
         data = fp.read()
-    meta = json.loads(data)
-    return meta
+    return json.loads(data)
 
 
 all_files_meta = get_all_sample_files()
@@ -148,7 +148,7 @@ def test_mediabox_expansion_after_rotation(
 
     mediabox = reader.pages[0].mediabox
 
-    # Deviation of upto 2 pixels is acceptable
+    # Deviation of up to 2 pixels is acceptable
     assert math.isclose(mediabox.width, expected_width, abs_tol=2)
     assert math.isclose(mediabox.height, expected_height, abs_tol=2)
 
@@ -400,12 +400,12 @@ def test_iss_1142():
     [
         # keyerror_potentially_empty_page
         (
-            "https://corpora.tika.apache.org/base/docs/govdocs1/964/964029.pdf",
+            "https://github.com/user-attachments/files/18381736/tika-964029.pdf",
             "tika-964029.pdf",
         ),
         # 1140 / 1141:
         (
-            "https://corpora.tika.apache.org/base/docs/govdocs1/932/932446.pdf",
+            "https://github.com/user-attachments/files/18381702/tika-932446.pdf",
             "tika-932446.pdf",
         ),
         # iss 1134:
@@ -419,7 +419,7 @@ def test_iss_1142():
             "WFCA.pdf",
         ),
         (
-            "https://corpora.tika.apache.org/base/docs/govdocs1/964/964029.pdf",
+            "https://github.com/user-attachments/files/18381736/tika-964029.pdf",
             "tika-964029.pdf",
         ),  # single_quote_op
         (
@@ -442,7 +442,7 @@ def test_extract_text(url, name):
 @pytest.mark.enable_socket
 @pytest.mark.slow
 def test_extract_text_page_pdf_impossible_decode_xform(caplog):
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/972/972962.pdf"
+    url = "https://github.com/user-attachments/files/18381748/tika-972962.pdf"
     name = "tika-972962.pdf"
     reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     for page in reader.pages:
@@ -454,7 +454,7 @@ def test_extract_text_page_pdf_impossible_decode_xform(caplog):
 @pytest.mark.enable_socket
 @pytest.mark.slow
 def test_extract_text_operator_t_star():  # L1266, L1267
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/967/967943.pdf"
+    url = "https://github.com/user-attachments/files/18381740/tika-967943.pdf"
     name = "tika-967943.pdf"
     reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     for page in reader.pages:
@@ -479,7 +479,7 @@ def test_extract_text_visitor_callbacks():
         """
 
         def __init__(self, text, x, y, font_dict, font_size) -> None:
-            # TODO \0-replace: Encoding issue in some files?
+            # TODO: \0-replace: Encoding issue in some files?
             self.text = text.replace("\0", "")
             self.x = x
             self.y = y
@@ -507,10 +507,8 @@ def test_extract_text_visitor_callbacks():
 
         def contains(self, x, y) -> bool:
             return (
-                x >= self.x
-                and x <= (self.x + self.w)
-                and y >= self.y
-                and y <= (self.y + self.h)
+                self.x <= x <= (self.x + self.w)
+                and self.y <= y <= (self.y + self.h)
             )
 
     def extract_text_and_rectangles(
@@ -943,7 +941,7 @@ def test_annotation_setter(pdf_file_path):
 @pytest.mark.enable_socket
 @pytest.mark.xfail(reason="#1091")
 def test_text_extraction_issue_1091():
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/966/966635.pdf"
+    url = "https://github.com/user-attachments/files/18381737/tika-966635.pdf"
     name = "tika-966635.pdf"
     stream = BytesIO(get_data_from_url(url, name=name))
     with pytest.warns(PdfReadWarning):
@@ -954,7 +952,7 @@ def test_text_extraction_issue_1091():
 
 @pytest.mark.enable_socket
 def test_empyt_password_1088():
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/941/941536.pdf"
+    url = "https://github.com/user-attachments/files/18381712/tika-941536.pdf"
     name = "tika-941536.pdf"
     stream = BytesIO(get_data_from_url(url, name=name))
     reader = PdfReader(stream)
@@ -1223,7 +1221,7 @@ def test_pages_printing():
 
 @pytest.mark.enable_socket
 def test_del_pages():
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/941/941536.pdf"
+    url = "https://github.com/user-attachments/files/18381712/tika-941536.pdf"
     name = "tika-941536.pdf"
     writer = PdfWriter(clone_from=BytesIO(get_data_from_url(url, name=name)))
     ll = len(writer.pages)
@@ -1459,6 +1457,8 @@ def test_get_contents_as_bytes():
     assert writer.pages[0]._get_contents_as_bytes() == expected
     writer.pages[0][NameObject("/Contents")] = writer.pages[0]["/Contents"][0]
     assert writer.pages[0]._get_contents_as_bytes() == expected
+    del writer.pages[0]["/Contents"]
+    assert writer.pages[0]._get_contents_as_bytes() is None
 
 
 def test_recursive_get_page_from_node():
@@ -1472,3 +1472,36 @@ def test_recursive_get_page_from_node():
     writer.insert_page(writer.pages[0], -1)
     with pytest.raises(ValueError):
         writer.insert_page(writer.pages[0], -10)
+
+
+def test_get_contents__none_type():
+    # We can observe this in reality as well, but these documents might be
+    # confidential. Thus use a more complex dummy implementation here while
+    # assigning a value of `None` is not possible from code, but from PDFs
+    # itself.
+    class MyPage(PageObject):
+        def __contains__(self, item) -> bool:
+            assert item == "/Contents"
+            return True
+
+        def __getitem__(self, item) -> Any:
+            assert item == "/Contents"
+
+    page = MyPage()
+    assert page.get_contents() is None
+
+
+def test_extract_text__none_type():
+    class MyPage(PageObject):
+        def __getitem__(self, item) -> Any:
+            if item == "/Contents":
+                return None
+            return super().__getitem__(item)
+
+    page = MyPage()
+    resources = DictionaryObject()
+    none_reference = IndirectObject(1, 0, None)
+    resources[NameObject("/Font")] = none_reference
+    page[NameObject("/Resources")] = resources
+    with mock.patch.object(none_reference, "get_object", return_value=None):
+        assert page.extract_text() == ""
